# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-07-29 21:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0002_user_manual_created_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='estimate_minutes',
            field=models.IntegerField(blank=True, null=True, verbose_name='logged minutes'),
        ),
        migrations.AddField(
            model_name='issue',
            name='logged_minutes',
            field=models.IntegerField(blank=True, null=True, verbose_name='logged minutes'),
        ),
    ]
